#!/usr/bin/env python
# -*- coding: euc-jp -*-

##
# @file Manager.py
# @brief RTComponent manager class
# @date $Date: $
# @author Noriaki Ando <n-ando@aist.go.jp> and Shinji Kurihara
#
# Copyright (C) 2006-2008
#     Task-intelligence Research Group,
#     Intelligent Systems Research Institute,
#     National Institute of
#         Advanced Industrial Science and Technology (AIST), Japan
#     All rights reserved.

import threading
import string
import signal, os
import sys
import traceback
import time
from omniORB import CORBA, PortableServer
from types import IntType, ListType

import OpenRTM
import RTC
import SDOPackage



#------------------------------------------------------------
# static var
#------------------------------------------------------------

##
# @if jp
# @brief ͣ Manager ؤΥݥ
# @else
# @brief The pointer to the Manager
# @endif
manager = None

##
# @if jp
# @brief ͣ Manager ؤΥݥ󥿤Ф mutex
# @else
# @brief The mutex of the pointer to the Manager 
# @endif
mutex = threading.RLock()


##
# @if jp
# @brief λ
#
# ޥ͡λ
#
# @param signum ʥֹ
# @param frame ߤΥåե졼
#
# @else
#
# @endif
def handler(signum, frame):
  mgr = OpenRTM.Manager.instance()
  mgr.terminate()



##
# @if jp
# @class ScopedLock
# @brief ScopedLock 饹
#
# ¾ѥå饹
#
# @since 0.4.0
#
# @else
#
# @endif
class ScopedLock:



  ##
  # @if jp
  # @brief 󥹥ȥ饯
  #
  # 󥹥ȥ饯
  #
  # @param self
  # @param mutex åѥߥ塼ƥå
  #
  # @else
  #
  # @endif
  def __init__(self, mutex):
    self.mutex = mutex
    self.mutex.acquire()


  ##
  # @if jp
  # @brief ǥȥ饯
  #
  # ǥȥ饯
  #
  # @param self
  #
  # @else
  #
  # @endif
  def __del__(self):
    self.mutex.release()



##
# @if jp
# @class Manager
# @brief Manager 饹
#
# ݡͥȤʤɳƼξԤޥ͡㥯饹
#
# @since 0.2.0
#
# @else
# @class Manager
# @brief Manager class
# @endif
class Manager:
  """
  """



  ##
  # @if jp
  # @brief ԡ󥹥ȥ饯
  #
  # ԡ󥹥ȥ饯
  #
  # @param self
  # @param _manager ԡޥ͡㥪֥(ǥե:None)
  #
  # @else
  # @brief Protected Copy Constructor
  #
  # @endif
  def __init__(self, _manager=None):
    self._initProc   = None
    self._runner     = None
    self._terminator = None
    self._compManager = OpenRTM.ObjectManager(self.InstanceName)
    self._factory = OpenRTM.ObjectManager(self.FactoryPredicate)
    self._ecfactory = OpenRTM.ObjectManager(self.ECFactoryPredicate)
    self._terminate = self.Term()
    self._ecs = []
    self._timer = None
    signal.signal(signal.SIGINT, handler)
    
    return


  ##
  # @if jp
  # @brief ޥ͡ν
  #
  # ޥ͡ static ؿ
  # ޥ͡򥳥ޥɥ饤Ϳƽ롣
  # ޥ͡Ѥϡɬνдؿ init() 
  # ƤФʤФʤʤ
  # ޥ͡Υ󥹥󥹤ˡȤơinit(), instance() 
  # 2Ĥ static ؿѰդƤ뤬init()ǤԤʤᡢ
  # Manager ¸֤ΰֺǽˤinit()Ƥɬפ롣
  #
  # ޥ͡ν
  # - initManager: configեɤ߹ߡ֥ƥ
  # - initLogger: Logger
  # - initORB: ORB 
  # - initNaming: NamingService 
  # - initExecutionContext: ExecutionContext factory 
  # - initTimer: Timer 
  #
  # @param argv ޥɥ饤
  # 
  # @return Manager ͣΥ󥹥󥹤λ
  #
  # @else
  # @brief Initializa manager
  #
  # This is the static function to tintialize the Manager.
  # The Manager is initialized by given arguments.
  # At the starting the manager, this static function "must" be called from
  # application program. The manager has two static functions to get 
  # the instance, "init()" and "instance()". Since initializing
  # process is only performed by the "init()" function, the "init()" has
  # to be called at the beginning of the lifecycle of the Manager.
  # function.
  #
  # @param argv The array of the command line arguments.
  #
  # @endif
  def init(*arg):
    global manager
    global mutex
    
    if len(arg) == 1:
      argv = arg[0]
    elif len(arg) == 2 and \
             isinstance(arg[0], IntType) and \
             isinstance(arg[1], ListType):
      argv = arg[1]
    else:
      print "Invalid arguments for init()"
      print "init(argc,argv) or init(argv)"
        
    if manager is None:
      guard = ScopedLock(mutex)
      if manager is None:
        manager = Manager()
        manager.initManager(argv)
        manager.initLogger()
        manager.initORB()
        manager.initNaming()
        manager.initExecContext()
        manager.initTimer()

    return manager
  
  init = staticmethod(init)


  ##
  # @if jp
  # @brief ޥ͡Υ󥹥󥹤μ
  #
  # ޥ͡Υ󥹥󥹤 static ؿ
  # δؿƤˡɬνؿ init() ƤФƤɬפ롣
  #
  # @return Manager ͣΥ󥹥󥹤λ
  # 
  # @else
  #
  # @brief Get instance of the manager
  #
  # This is the static function to get the instance of the Manager.
  # Before calling this function, ensure that the initialization function
  # "init()" is called.
  #
  # @return The only instance reference of the manager
  #
  # @endif
  def instance():
    global manager
    global mutex
    
    if manager is None:
      guard = ScopedLock(mutex)
      if manager is None:
        manager = Manager()
        manager.initManager(None)
        manager.initLogger()
        manager.initORB()
        manager.initNaming()
        manager.initExecContext()
        manager.initTimer()

    return manager

  instance = staticmethod(instance)


  ##
  # @if jp
  # @brief ޥ͡㽪λ
  #
  # ޥ͡νλ¹Ԥ롣
  #
  # @param self
  #
  # @else
  #
  # @endif
  def terminate(self):
    if self._terminator:
      self._terminator.terminate()


  ##
  # @if jp
  # @brief ޥ͡㡦åȥ
  #
  # ޥ͡νλ¹Ԥ롣
  # ORBλ塢Ʊäƽλ롣
  #
  # @param self
  #
  # @else
  #
  # @endif
  def shutdown(self):
    self._rtcout.RTC_DEBUG("Manager::shutdown()")
    self.shutdownComponents()
    self.shutdownNaming()
    self.shutdownORB()
    self.shutdownManager()

    if self._runner:
      self._runner.wait()
    else:
      self.join()

    self.shutdownLogger()


  ##
  # @if jp
  # @brief ޥ͡㽪λԤ碌
  #
  # Ʊ뤿ᡢޥ͡㽪λԤ碌Ԥ
  #
  # @param self
  #
  # @else
  #
  # @endif
  def join(self):
    self._rtcout.RTC_DEBUG("Manager::wait()")
    guard = ScopedLock(self._terminate.mutex)
    self._terminate.waiting += 1
    del guard
    while 1:
      guard = ScopedLock(self._terminate.mutex)
      #if self._terminate.waiting > 1:
      if self._terminate.waiting > 0:
        break
      del guard
      time.sleep(0.001)


  ##
  # @if jp
  #
  # @brief ץΥå
  #
  # Υڥ졼ϥ桼Ԥ⥸塼νץ
  # ꤹ롣ꤵ줿ץϡޥ͡㤬졢
  # ƥֲ줿塢Ŭڤʥߥ󥰤Ǽ¹Ԥ롣
  #
  # @param self
  # @param proc ץδؿݥ
  #
  # @else
  #
  # @brief Run the Manager
  #
  # This operation sets the initial procedure call to process module
  # initialization, other user defined initialization and so on.
  # The given procedure will be called at the proper timing after the 
  # manager initialization, activation and run.
  #
  # @param proc A function pointer to the initial procedure call
  #
  # @endif
  def setModuleInitProc(self, proc):
    self._initProc = proc
    return


  ##
  # @if jp
  #
  # @brief ManagerΥƥֲ
  #
  # Υڥ졼ϰʲνԤ
  # - CORBA POAManager Υƥֲ
  # - ޥ͡CORBA֥ȤΥƥֲ
  # - Manager ֥Ȥؤνץμ¹
  #
  # Υڥ졼ϡޥ͡ν塢runManager()
  # ˸Ƥɬפ롣
  #
  # @param self
  #
  # @return (ƥֲ:true:false)
  #
  # @else
  #
  # @brief Activate Manager
  #
  # This operation do the following,
  # - Activate CORBA POAManager
  # - Activate Manager CORBA object
  # - Execute the initial procedure call of the Manager
  #
  # This operationo should be invoked after Manager:init(),
  # and before tunManager().
  #
  # @endif
  def activateManager(self):
    self._rtcout.RTC_DEBUG("Manager::activateManager()")

    try:
      self.getPOAManager().activate()
      if self._initProc:
        self._initProc(self)
    except:
      print "Exception: Manager.activateManager()"
      return False

    return True


  ##
  # @if jp
  #
  # @brief Managerμ¹
  #
  # Υڥ졼ϥޥ͡Υᥤ롼פ¹Ԥ롣
  # Υᥤ롼ǤϡCORBA ORBΥ٥ȥ롼
  # 롣ǥեȤǤϡΥڥ졼ϥ֥å
  # Manager::destroy() ƤФޤǽᤵʤ
  #  no_block  true ꤵƤϡǥ٥ȥ롼
  # 륹åɤư֥å˽᤹
  #
  # @param self
  # @param no_block false: ֥å󥰥⡼, true: Υ֥å󥰥⡼
  #
  # @else
  #
  # @brief Run the Manager
  #
  # This operation processes the main event loop of the Manager.
  # In this main loop, CORBA's ORB event loop or other processes
  # are performed. As the default behavior, this operation is going to
  # blocking mode and never returns until manager::destroy() is called.
  # When the given argument "no_block" is set to "true", this operation
  # creates a thread to process the event loop internally, and it doesn't
  # block and returns.
  #
  # @param no_block false: Blocking mode, true: non-blocking mode.
  #
  # @endif
  def runManager(self, no_block=None):
    if no_block is None:
      no_block = False

    if no_block:
      self._rtcout.RTC_DEBUG("Manager::runManager(): non-blocking mode")
      self._runner = self.OrbRunner(self._orb)
      # self._runnner.open()
    else:
      self._rtcout.RTC_DEBUG("Manager::runManager(): blocking mode")
      self._orb.run()
      self._rtcout.RTC_DEBUG("Manager::runManager(): ORB was terminated")
      self.join()
    return


  ##
  # @if jp
  # @brief [CORBA interface] ⥸塼Υ
  #
  # ꤷݡͥȤΥ⥸塼ɤȤȤˡ
  # ꤷؿ¹Ԥ롣
  #
  # @param self
  # @param fname   ⥸塼ե̾
  # @param initfunc ؿ̾
  # 
  # @else
  #
  # @brief [CORBA interface] Load module
  #
  # Load module (shared library, DLL etc..) by file name,
  # and invoke initialize function.
  #
  # @param fname    The module file name
  # @param initfunc The initialize function name
  #
  # @endif
  def load(self, fname, initfunc):
    self._rtcout.RTC_DEBUG("Manager::load()")
    self._module.load(fname, initfunc)
    return


  ##
  # @if jp
  #
  # @brief ⥸塼Υ
  #
  # ⥸塼򥢥ɤ
  #
  # @param self
  # @param fname ⥸塼Υե̾
  # 
  # @else
  #
  # @brief Unload module
  #
  # Unload shared library.
  #
  # @param pathname Module file name
  #
  # @endif
  def unload(self, fname):
    self._rtcout.RTC_DEBUG("Manager::unload()")
    self._module.unload(fname)
    return


  ##
  # @if jp
  #
  # @brief ⥸塼Υ
  #
  # ⥸塼򤹤٤ƥɤ
  #
  # @param self
  #
  # @else
  #
  # @brief Unload module
  #
  # Unload all loaded shared library.
  #
  # @endif
  def unloadAll(self):
    self._rtcout.RTC_DEBUG("Manager::unloadAll()")
    self._module.unloadAll()
    return


  ##
  # @if jp
  # @brief ɺѤߤΥ⥸塼ꥹȤ
  #
  # ߥޥ͡˥ɤƤ⥸塼ΥꥹȤ롣
  #
  # @param self
  #
  # @return ɺѤߥ⥸塼ꥹ
  #
  # @else
  # @brief Get loaded module names
  # @endif
  def getLoadedModules(self):
    self._rtcout.RTC_DEBUG("Manager::getLoadedModules()")
    return self._module.getLoadedModules()


  ##
  # @if jp
  # @brief ɲǽʥ⥸塼ꥹȤ
  #
  # ɲǽ⥸塼ΥꥹȤ롣
  # (ߤModuleManager¦̤)
  #
  # @param self
  #
  # @return ɲǽ⥸塼롡ꥹ
  #
  # @else
  # @brief Get loadable module names
  # @endif
  def getLoadableModules(self):
    self._rtcout.RTC_DEBUG("Manager::getLoadableModules()")
    return self._module.getLoadableModules()


  #============================================================
  # Component Factory Management
  #============================================================

  ##
  # @if jp
  # @brief RTݡͥѥեȥϿ
  #
  # RTݡͥȤΥ󥹥󥹤뤿
  # FactoryϿ롣
  #
  # @param self
  # @param profile RTݡͥ ץե
  # @param new_func RTݡͥѴؿ
  # @param delete_func RTݡͥ˴Ѵؿ
  #
  # @return Ͽ(Ͽ:true:false)
  #
  # @else
  # @brief Register RT-Component Factory
  # @endif
  def registerFactory(self, profile, new_func, delete_func):
    self._rtcout.RTC_DEBUG("Manager::registerFactory(%s)", profile.getProperty("type_name"))
    try:
      factory = OpenRTM.FactoryPython(profile, new_func, delete_func)
      self._factory.registerObject(factory)
      return True
    except:
      return False


  ##
  # @if jp
  # @brief ExecutionContextѥեȥϿ
  #
  # ExecutionContextΥ󥹥󥹤뤿FactoryϿ롣
  #
  # @param self
  # @param name оExecutionContext̾
  # @param new_func ExecutionContextѴؿ
  # @param delete_func ExecutionContext˴Ѵؿ
  #
  # @return Ͽ(Ͽ:true:false)
  #
  # @else
  # @brief Register ExecutionContext Factory
  # @endif
  def registerECFactory(self, name, new_func, delete_func):
    self._rtcout.RTC_DEBUG("Manager::registerECFactory(%s)", name)
    try:
      self._ecfactory.registerObject(OpenRTM.ECFactoryPython(name, new_func, delete_func))
      return True
    except:
      return False

    return False


  ##
  # @if jp
  # @brief եȥꥹȤ
  #
  # ϿƤեȥꥹȤ롣
  #
  # @param self
  #
  # @return Ͽեȥ ꥹ
  #
  # @else
  # @brief Get the list of all RT-Component Factory
  # @endif
  def getModulesFactories(self):
    self._rtcout.RTC_DEBUG("Manager::getModulesFactories()")

    self._modlist = []
    for _obj in self._factory._objects._obj:
      self._modlist.append(_obj.profile().getProperty("implementation_id"))
    return self._modlist


  #============================================================
  # Component management
  #============================================================

  ##
  # @if jp
  # @brief RTݡͥȤ
  #
  # ꤷRTݡͥȤΥ󥹥󥹤Ͽ줿Factoryͳ롣
  # 󥹥硢ʻưʲν¹Ԥ롣
  #  - եꤷե졼ɤ߹ߡ
  #  - ExecutionContextΥХɡư
  #  - ͡ߥ󥰥ӥؤϿ
  #
  # @param self
  # @param module_name оRTݡͥ̾
  #
  # @return RTݡͥȤΥ󥹥
  #
  # @else
  # @brief Create RT-Component
  # @endif
  def createComponent(self, module_name):
    self._rtcout.RTC_DEBUG("Manager::createComponent(%s)", module_name)

    obj = self._factory.find(module_name)
    comp = obj.create(self)
    if comp is None:
      return None
    self._rtcout.RTC_DEBUG("RTC Created: %s", module_name)

    self.configureComponent(comp)

    if comp.initialize() != RTC.RTC_OK:
      self._rtcout.RTC_DEBUG("RTC initialization failed: %s", module_name)
      comp.exit()
      self._rtcout.RTC_DEBUG("%s was finalized", module_name)
      return None

    self._rtcout.RTC_DEBUG("RTC initialization succeeded: %s", module_name)
    self.bindExecutionContext(comp)
    self.registerComponent(comp)
    return comp


  ##
  # @if jp
  # @brief RTݡͥȤľ Manager Ͽ
  #
  # ꤷRTݡͥȤΥ󥹥󥹤եȥͳǤϤʤ
  # ľܥޥ͡Ͽ롣
  #
  # @param self
  # @param comp ϿоRTݡͥȤΥ󥹥
  #
  # @return Ͽ(Ͽ:true:false)
  #
  # @else
  # @brief Register RT-Component directly without Factory
  # @endif
  def registerComponent(self, comp):
    self._rtcout.RTC_DEBUG("Manager::registerComponent(%s)", comp.getInstanceName())

    self._compManager.registerObject(comp)
    names = comp.getNamingNames()

    for name in names:
      self._rtcout.RTC_DEBUG("Bind name: %s", name)
      self._namingManager.bindObject(name, comp)

    return True

  
  ##
  # @if jp
  # @brief RTݡͥȤϿ
  #
  # ꤷRTݡͥȤϿ롣
  #
  # @param self
  # @param comp ϿоRTݡͥȤΥ󥹥
  #
  # @return Ͽ(:true:false)
  #
  # @else
  # @brief Register RT-Component directly without Factory
  # @endif
  def unregisterComponent(self, comp):
    self._rtcout.RTC_DEBUG("Manager::unregisterComponent(%s)", comp.getInstanceName())
    self._compManager.unregisterObject(comp.getInstanceName())
    names = comp.getNamingNames()
    
    for name in names:
      self._rtcout.RTC_DEBUG("Unbind name: %s", name)
      self._namingManager.unbindObject(name)

    return True


  ##
  # @if jp
  # @brief RTݡͥȤExecutionContextХɤ
  #
  # ꤷRTݡͥȤExecutionContextХɤ롣
  # ХɤExecutionContextηϥץѥƥե
  # "exec_cxt.periodic.type"°ˤäƻꤹ롣
  #
  # @param self
  # @param comp ХоRTݡͥȤΥ󥹥
  #
  # @return Хɽ(Х:true:false)
  #
  # @else
  # @brief Register RT-Component directly without Factory
  # @endif
  def bindExecutionContext(self, comp):
    self._rtcout.RTC_DEBUG("Manager::bindExecutionContext()")
    self._rtcout.RTC_DEBUG("ExecutionContext type: %s",
                 self._config.getProperty("exec_cxt.periodic.type"))

    rtobj = comp.getObjRef()

    exec_cxt = None

    if OpenRTM.isDataFlowParticipant(rtobj):
      ectype = self._config.getProperty("exec_cxt.periodic.type")
      exec_cxt = self._ecfactory.find(ectype).create()
      rate = self._config.getProperty("exec_cxt.periodic.rate")
      exec_cxt.set_rate(float(rate))
    else:
      ectype = self._config.getProperty("exec_cxt.evdriven.type")
      exec_cxt = self._ecfactory.find(ectype).create()
    exec_cxt.add(rtobj)
    exec_cxt.start()
    self._ecs.append(exec_cxt)
    return True


  ##
  # @if jp
  # @brief Manager ϿƤRTݡͥȤ(̤)
  #
  # ޥ͡ϿƤRTݡͥȤ롣
  #
  # @param self
  # @param instance_name оRTݡͥȤΥ󥹥̾
  #
  # @else
  # @brief Unregister RT-Component that is registered in the Manager
  # @endif
  def deleteComponent(self, instance_name):
    self._rtcout.RTC_DEBUG("Manager::deleteComponent(%s)", instance_name)


  ##
  # @if jp
  # @brief Manager ϿƤRTݡͥȤ򸡺
  #
  # Manager ϿƤRTݡͥȤꤷ̾ΤǸ
  # פ륳ݡͥȤ롣
  #
  # @param self
  # @param instance_name оRTݡͥȤ̾
  #
  # @return ̾ΤפRTݡͥȤΥ󥹥
  #
  # @else
  # @brief Get RT-Component's pointer
  # @endif
  def getComponent(self, instance_name):
    self._rtcout.RTC_DEBUG("Manager::getComponent(%s)", instance_name)
    return self._compManager.find(instance_name)


  ##
  # @if jp
  # @brief Manager ϿƤRTݡͥȤ
  #
  # Manager ϿƤRTݡͥȤ󥹥󥹤롣
  #
  # @param self
  #
  # @return RTݡͥȤΥ󥹥󥹥ꥹ
  #
  # @else
  # @brief Get all RT-Component's pointer
  # @endif
  def getComponents(self):
    self._rtcout.RTC_DEBUG("Manager::getComponents()")
    return self._compManager.getObjects()


  #============================================================
  # CORBA Ϣ
  #============================================================

  ##
  # @if jp
  # @brief ORB Υݥ󥿤
  #
  # Manager ꤵ줿 ORB Υݥ󥿤롣
  #
  # @param self
  #
  # @return ORB ֥
  #
  # @else
  # @brief Get the pointer to the ORB
  # @endif
  def getORB(self):
    self._rtcout.RTC_DEBUG("Manager::getORB()")
    return self._orb


  ##
  # @if jp
  # @brief Manager  RootPOA Υݥ󥿤
  #
  # Manager ꤵ줿 RootPOA ؤΥݥ󥿤롣
  #
  # @param self
  #
  # @return RootPOA֥
  #
  # @else
  # @brief Get the pointer to the RootPOA 
  # @endif
  def getPOA(self):
    self._rtcout.RTC_DEBUG("Manager::getPOA()")
    return self._poa


  ##
  # @if jp
  # @brief Manager  POAManager 
  #
  # Manager ꤵ줿 POAMAnager 롣
  #
  # @param self
  #
  # @return POAޥ͡
  #
  # @else
  #
  # @endif
  def getPOAManager(self):
    self._rtcout.RTC_DEBUG("Manager::getPOAManager()")
    return self._poaManager



  #============================================================
  # Manager initialize and finalization
  #============================================================

  ##
  # @if jp
  # @brief Manager 
  # 
  # Manager ¹Ԥ롣
  #  - Manager ե졼
  #  - ϥե
  #  - λѥåɤ
  #  - ѥåɤ(޻ѻ)
  #
  # @param self
  # @param argv ޥɥ饤
  # 
  # @else
  # @brief Manager internal initialization
  # @endif
  def initManager(self, argv):
    config = OpenRTM.ManagerConfig(argv)
    self._config = config.configure(OpenRTM.Properties())
    self._config.setProperty("logger.file_name",self.formatString(self._config.getProperty("logger.file_name"), self._config))

    self._module = OpenRTM.ModuleManager(self._config)
    self._terminator = self.Terminator(self)
    guard = ScopedLock(self._terminate.mutex)
    self._terminate.waiting = 0
    del guard

    if OpenRTM.toBool(self._config.getProperty("timer.enable"), "YES", "NO", True):
      tm = OpenRTM.TimeValue(0, 100000)
      tick = self._config.getProperty("timer.tick")
      if tick != "":
        tm = tm.set_time(float(tick))
        self._timer = OpenRTM.Timer(tm)
        self._timer.start()


  ##
  # @if jp
  # @brief Manager νλ(̤)
  #
  # Manager λ
  # (ߤ̤)
  #
  # @param self
  #
  # @else
  #
  # @endif
  def shutdownManager(self):
    self._rtcout.RTC_DEBUG("Manager::shutdownManager()")
    if self._timer:
      self._timer.stop()


  #============================================================
  # Logger initialize and terminator
  #============================================================

  ##
  # @if jp
  # @brief System logger ν
  #
  # System logger ν¹Ԥ롣
  # ե졼եꤵ줿˴Ť
  # ν¹Ԥ롣
  #
  # @param self
  #
  # @return ¹Է(:true:false)
  #
  # @else
  # @brief System logger initialization
  # @endif
  def initLogger(self):
    logfile = self._config.getProperty("logger.file_name")
    if logfile == "":
      logfile = "./rtc.log"

    if OpenRTM.toBool(self._config.getProperty("logger.enable"), "YES", "NO", True):
      self._Logbuf = OpenRTM.Logbuf(fileName = logfile)
      self._rtcout = OpenRTM.LogStream(self._Logbuf)
      self._rtcout.setLogLevel(self._config.getProperty("logger.log_level"))
      self._rtcout.setLogLock(OpenRTM.toBool(self._config.getProperty("logger.stream_lock"),
                          "enable", "disable", False))

      self._rtcout.RTC_INFO("%s", self._config.getProperty("openrtm.version"))
      self._rtcout.RTC_INFO("Copyright (C) 2003-2007")
      self._rtcout.RTC_INFO("  Noriaki Ando")
      self._rtcout.RTC_INFO("  Task-intelligence Research Group,")
      self._rtcout.RTC_INFO("  Intelligent Systems Research Institute, AIST")
      self._rtcout.RTC_INFO("Manager starting.")
      self._rtcout.RTC_INFO("Starting local logging.")
    else:
      self._rtcout = OpenRTM.LogStream()

    return True


  ##
  # @if jp
  # @brief System Logger νλ(̤)
  #
  # System Loggerνλ¹Ԥ롣
  # (ߤ̤)
  #
  # @param self
  #
  # @else
  # @brief System Logger finalization
  # @endif
  def shutdownLogger(self):
    self._rtcout.RTC_DEBUG("Manager::shutdownLogger()")


  #============================================================
  # ORB initialization and finalization
  #============================================================

  ##
  # @if jp
  # @brief CORBA ORB ν
  #
  # 򸵤ORB롣
  #
  # @param self
  #
  # @return ORB (:true:false)
  #
  # @else
  # @brief CORBA ORB initialization
  # @endif
  def initORB(self):
    self._rtcout.RTC_DEBUG("Manager::initORB()")

    try:
      args = OpenRTM.split(self.createORBOptions(), " ")
      argv = OpenRTM.toArgv(args)
      self._orb = CORBA.ORB_init(argv)

      self._poa = self._orb.resolve_initial_references("RootPOA")

      if CORBA.is_nil(self._poa):
        self._rtcout.RTC_ERROR("Could not resolve RootPOA")
        return False

      self._poaManager = self._poa._get_the_POAManager()
      self._objManager = OpenRTM.CorbaObjectManager(self._orb, self._poa)
    except:
      self._rtcout.RTC_ERROR("Exception: Caught unknown exception in initORB().")
      return False

    return True


  ##
  # @if jp
  # @brief ORB Υޥɥ饤󥪥ץ
  #
  # ե졼ꤵ줿Ƥ
  # ORB εưץ롣
  #
  # @param self
  #
  # @return ORB ưץ
  #
  # @else
  # @brief ORB command option creation
  # @endif
  def createORBOptions(self):
    opt      = self._config.getProperty("corba.args")
    corba    = self._config.getProperty("corba.id")
    endpoint = self._config.getProperty("corba.endpoint")

    if endpoint != "":
      if opt != "":
        opt += " "
      if corba == "omniORB":
        opt = "-ORBendPoint giop:tcp:" + endpoint
      elif corba == "TAO":
        opt = "-ORBEndPoint iiop://" + endpoint
      elif corba == "MICO":
        opt = "-ORBIIOPAddr inet:" + endpoint
    return opt


  ##
  # @if jp
  # @brief ORB νλ
  #
  # ORB νλ¹Ԥ롣
  # ¹Ԥν¸ߤˤϡνλޤԤġ
  # ºݤνλǤϡPOA Manager ORB Υåȥ¹
  # 롣
  #
  # @param self
  #
  # @else
  # @brief ORB finalization
  # @endif
  def shutdownORB(self):
    self._rtcout.RTC_DEBUG("Manager::shutdownORB()")
    try:
      while self._orb.work_pending():
        self._rtcout.RTC_PARANOID("Pending work still exists.")
        if self._orb.work_pending():
          self._orb.perform_work()
    except:
      traceback.print_exception(*sys.exc_info())
      pass

    self._rtcout.RTC_DEBUG("No pending works of ORB. Shutting down POA and ORB.")

    if not CORBA.is_nil(self._poa):
      try:
        if not CORBA.is_nil(self._poaManager):
          self._poaManager.deactivate(False, True)
        self._rtcout.RTC_DEBUG("POA Manager was deactivated.")
        self._poa.destroy(False, True)
        self._poa = PortableServer.POA._nil
        self._rtcout.RTC_DEBUG("POA was destroid.")
      except CORBA.SystemException, ex:
        self._rtcout.RTC_ERROR("Caught SystemException during root POA destruction")
      except:
        self._rtcout.RTC_ERROR("Caught unknown exception during destruction")

    if self._orb:
      try:
        self._orb.shutdown(True)
        self._rtcout.RTC_DEBUG("ORB was shutdown.")
        self._orb = CORBA.Object._nil
      except CORBA.SystemException, ex:
        self._rtcout.RTC_ERROR("Caught CORBA::SystemException during ORB shutdown.")
      except:
        self._rtcout.RTC_ERROR("Caught unknown exception during ORB shutdown.")


  #============================================================
  # NamingService initialization and finalization
  #============================================================

  ##
  # @if jp
  # @brief NamingManager ν
  #
  # NamingManager ν¹Ԥ롣
  #  NamingManager Ѥʤ褦˥ץѥƥꤵƤ
  # ˤϲ⤷ʤ
  # NamingManager Ѥ硢ץѥƥꤵƤ
  # ǥե NamingServer Ͽ롣
  # ޤŪ˾򹹿褦ꤵƤˤϡꤵ줿
  # ǼưԤΥޤưȤȤˡѥ᥽åɤ򥿥ޤ
  # Ͽ롣
  #
  # @param self
  #
  # @return (:true:false)
  #
  # @else
  #
  # @endif
  def initNaming(self):
    self._rtcout.RTC_DEBUG("Manager::initNaming()")
    self._namingManager = OpenRTM.NamingManager(self)

    if not OpenRTM.toBool(self._config.getProperty("naming.enable"), "YES", "NO", True):
      return True

    meths = OpenRTM.split(self._config.getProperty("naming.type"),",")
    
    for meth in meths:
      names = OpenRTM.split(self._config.getProperty(meth+".nameservers"), ",")
      for name in names:
        self._rtcout.RTC_DEBUG("Register Naming Server: %s/%s", (meth, name))
        self._namingManager.registerNameServer(meth,name)

    if OpenRTM.toBool(self._config.getProperty("naming.update.enable"), "YES", "NO", True):
      tm = OpenRTM.TimeValue(10,0)
      intr = self._config.getProperty("naming.update.interval")
      if intr != "":
        tm = OpenRTM.TimeValue(intr)

      if self._timer:
        self._timer.registerListenerObj(self._namingManager,OpenRTM.NamingManager.update,tm)
  
    return True


  ##
  # @if jp
  # @brief NamingManager νλ
  #
  # NamingManager λ롣
  # ϿƤǤ򥢥Хɤλ롣
  #
  # @param self
  #
  # @else
  #
  # @endif
  def shutdownNaming(self):
    self._rtcout.RTC_DEBUG("Manager::shutdownNaming()")
    self._namingManager.unbindAll()


  ##
  # @if jp
  # @brief ExecutionContextManager ν
  #
  # Ѥ ExecutionContext ν¹Ԥ ExecutionContext 
  #  Factory  ExecutionContextManager Ͽ롣
  #
  # @param self
  #
  # @return ExecutionContextManager ¹Է
  #         (:true:false)
  #
  # @else
  #
  # @endif
  def initExecContext(self):
    self._rtcout.RTC_DEBUG("Manager::initExecContext()")
    OpenRTM.PeriodicExecutionContextInit(self)
    OpenRTM.ExtTrigExecutionContextInit(self)
    return True


  ##
  # @if jp
  # @brief Timer ν
  #
  # Ѥ Timer ν¹Ԥ롣
  # (μǤϲ⤷ʤ)
  #
  # @param self
  #
  # @return Timer ¹Է(:true:false)
  #
  # @else
  #
  # @endif
  def initTimer(self):
    return True


  ##
  # @if jp
  # @brief NamingManager ϿƤݡͥȤνλ
  #
  # NamingManager ϿƤRTݡͥȤ ExecutionContext 
  # ꥹȤݡͥȤλ롣
  #
  # @param self
  #
  # @else
  #
  # @endif
  def shutdownComponents(self):
    self._rtcout.RTC_DEBUG("Manager::shutdownComponents()")
    comps = self._namingManager.getObjects()
    for comp in comps:
      try:
        comp.exit()
        p = OpenRTM.Properties(key=comp.getInstanceName())
        p.mergeProperties(comp.getProperties())
      except:
        traceback.print_exception(*sys.exc_info())
        pass

    for ec in self._ecs:
      try:
        self._poa.deactivate_object(self._poa.servant_to_id(ec))
      except:
        traceback.print_exception(*sys.exc_info())
        pass


  ##
  # @if jp
  # @brief RTݡͥȤϿ
  #
  # ꤷRTݡͥȤΥ󥹥󥹤͡ߥ󥰥ӥ
  # Ͽ롣
  #
  # @param self
  # @param comp ϿоRTݡͥ
  #
  # @else
  #
  # @endif
  def cleanupComponent(self, comp):
    self._rtcout.RTC_DEBUG("Manager::cleanupComponents")
    self.unregisterComponent(comp)


  ##
  # @if jp
  # @brief RTݡͥȤΥե졼
  #
  # RTݡͥȤηӥ󥹥˵ܤ줿ץѥƥե
  # ɤ߹ߡݡͥȤꤹ롣
  # ޤƥݡͥȤ NamingService Ͽ̾Τꤹ롣
  #
  # @param self
  # @param comp ե졼оRTݡͥ
  #
  # @else
  #
  # @endif
  def configureComponent(self, comp):
    category  = comp.getCategory()
    type_name = comp.getTypeName()
    inst_name = comp.getInstanceName()

    type_conf = category + "." + type_name + ".config_file"
    name_conf = category + "." + inst_name + ".config_file"

    type_prop = OpenRTM.Properties()

    name_prop = OpenRTM.Properties()

    if self._config.getProperty(name_conf) != "":
      try:
        conff = open(self._config.getProperty(name_conf))
      except:
        print "Not found. : %s" % self._config.getProperty(name_conf)
      else:
        name_prop.load(conff)

    if self._config.getProperty(type_conf) != "":
      try:
        conff = open(self._config.getProperty(type_conf))
      except:
        print "Not found. : %s" % self._config.getProperty(type_conf)
      else:
        type_prop.load(conff)

    type_prop = type_prop.mergeProperties(name_prop)
    comp.getProperties().mergeProperties(type_prop)

    naming_formats = ""
    comp_prop = OpenRTM.Properties(prop=comp.getProperties())

    naming_formats += self._config.getProperty("naming.formats")
    naming_formats += ", " + comp_prop.getProperty("naming.formats")

    naming_formats = OpenRTM.flatten(OpenRTM.unique_sv(OpenRTM.split(naming_formats, ",")))

    naming_names = self.formatString(naming_formats, comp.getProperties())
    comp.getProperties().setProperty("naming.formats",naming_formats)
    comp.getProperties().setProperty("naming.names",naming_names)


  ##
  # @if jp
  # @brief ץѥƥΥޡ
  #
  # ꤵ줿եꤵƤץѥƥɤ
  # ¸Ѥߥץѥƥȥޡ롣
  #
  # @param self
  # @param prop ޡоݥץѥƥ
  # @param file_name ץѥƥ󤬵ҤƤե̾
  #
  # @return ޡ¹Է(ޡ:trueޡ:false)
  #
  # @else
  #
  # @endif
  def mergeProperty(self, prop, file_name):
    if file_name == "":
      self._rtcout.RTC_ERROR("Invalid configuration file name.")
      return False

    if file_name[0] != '\0':
      
      try:
        conff = open(file_name)
      except:
        print "Not found. : %s" % file_name
      else:
        prop.load(conff)
        conff.close()
        return True

    return False

  ##
  # @if jp
  # @brief NamingServer ϿݤϿȤΩƤ
  #
  # ꤵ줿񼰤ȥץѥƥ NameServer Ͽݤξ
  # ȤΩƤ롣
  # ƽ񼰻ʸΰ̣ϰʲΤȤ
  # - % : ƥȤζڤ
  # - n : 󥹥̾
  # - t : ̾
  # - m : ̾
  # - v : С
  # - V : ٥
  # - c : ƥ
  # - h : ۥ̾
  # - M : ޥ̾͡
  # - p : ץID
  #
  # @param self
  # @param naming_format NamingService Ͽ񼰻
  # @param prop Ѥץѥƥ
  #
  # @return Ѵ
  #
  # @else
  #
  # @endif
  def formatString(self, naming_format, prop):
    name_ = naming_format
    str_  = ""
    count = 0

    for n in name_:
      if n == '%':
        count+=1
        if not (count % 2):
          str_ += n
      else:
        if  count > 0 and (count % 2):
          count = 0
          if   n == "n": str_ += prop.getProperty("instance_name")
          elif n == "t": str_ += prop.getProperty("type_name")
          elif n == "m": str_ += prop.getProperty("type_name")
          elif n == "v": str_ += prop.getProperty("version")
          elif n == "V": str_ += prop.getProperty("vendor")
          elif n == "c": str_ += prop.getProperty("category")
          elif n == "h": str_ += self._config.getProperty("manager.os.hostname")
          elif n == "M": str_ += self._config.getProperty("manager.name")
          elif n == "p": str_ += str(self._config.getProperty("manager.pid"))
          else: str_ += n
        else:
          count = 0
          str_ += n

    return str_


  ##
  # @if jp
  # @brief Хåեμ
  #
  # ޥ͡ꤷХåե롣
  #
  # @param self
  #
  # @return ޥ͡ꤷХåե
  #
  # @else
  #
  # @endif
  def getLogbuf(self):
    return self._rtcout


  ##
  # @if jp
  # @brief ޥ͡㥳ե졼μ
  #
  # ޥ͡ꤷե졼롣
  #
  # @param self
  #
  # @return ޥ͡Υե졼
  #
  # @else
  #
  # @endif
  def getConfig(self):
    return self._config


  #============================================================
  # ݡͥȥޥ͡
  #============================================================
  ##
  # @if jp
  # @class InstanceName
  # @brief ObjectManager ѥե󥯥
  #
  # @else
  #
  # @endif
  class InstanceName:



    ##
    # @if jp
    # @brief 󥹥ȥ饯
    #
    # 󥹥ȥ饯
    #
    # @param self
    # @param name оݥݡͥ̾(ǥե:None)
    # @param factory оݥեȥ̾(ǥե:None)
    #
    # @else
    #
    # @endif
    def __init__(self, name=None, factory=None):
      if factory:
        self._name = factory.getInstanceName()
      elif name:
        self._name = name

    def __call__(self, factory):
      return self._name == factory.getInstanceName()



  #============================================================
  # ݡͥȥեȥ
  #============================================================
  ##
  # @if jp
  # @class FactoryPredicate
  # @brief ݡͥȥեȥ긡ѥե󥯥
  #
  # @else
  #
  # @endif
  class FactoryPredicate:



    def __init__(self, name=None, factory=None):
      if name:
        self._name = name
      elif factory:
        self._name = factory.profile().getProperty("implementation_id")

    def __call__(self, factory):
      return self._name == factory.profile().getProperty("implementation_id")



  #============================================================
  # ExecutionContextեȥ
  #============================================================
  ##
  # @if jp
  # @class FactoryPredicate
  # @brief ExecutionContextեȥ긡ѥե󥯥
  #
  # @else
  #
  # @endif
  class ECFactoryPredicate:



    def __init__(self, name=None, factory=None):
      if name:
        self._name = name
      elif factory:
        self._name = factory.name()

    def __call__(self, factory):
      return self._name == factory.name()


  #------------------------------------------------------------
  # ORB runner
  #------------------------------------------------------------
  ##
  # @if jp
  # @class OrbRunner
  # @brief OrbRunner 饹
  #
  # ORB ¹ѥإѡ饹
  #
  # @since 0.4.0
  #
  # @else
  # @class OrbRunner
  # @brief OrbRunner class
  # @endif
  class OrbRunner:



    ##
    # @if jp
    # @brief 󥹥ȥ饯
    #
    # 󥹥ȥ饯
    #
    # @param self
    # @param orb ORB
    #
    # @else
    # @brief Constructor
    #
    # @endif
    def __init__(self, orb):
      self._orb = orb
      self._th = threading.Thread(target=self.run)
      self._th.start()
      self._evt = threading.Event()


    ##
    # @if jp
    # @brief ORB ¹Խ
    #
    # ORB ¹
    #
    # @param self
    #
    # @else
    #
    # @endif
    def run(self):
      try:
        self._orb.run()
        #Manager.instance().shutdown()
        self._evt.set()
      except:
        traceback.print_exception(*sys.exc_info())
        pass
      self._evt.set()
      return


    ##
    # @if jp
    # @brief ORB wait
    #
    # ORB wait
    #
    # @param self
    #
    # @else
    #
    # @endif
    def wait(self):
      self._evt.wait()

    ##
    # @if jp
    # @brief ORB λ(̤)
    #
    # ORB λ
    #
    # @param self
    # @param flags λե饰
    #
    # @return λ
    #
    # @else
    #
    # @endif
    def close(self, flags):
      return 0


  #------------------------------------------------------------
  # Manager Terminator
  #------------------------------------------------------------
  ##
  # @if jp
  # @class Terminator
  # @brief Terminator 饹
  #
  # ORB λѥإѡ饹
  #
  # @since 0.4.0
  #
  # @else
  #
  # @endif
  class Terminator:



    ##
    # @if jp
    # @brief 󥹥ȥ饯
    #
    # 󥹥ȥ饯
    #
    # @param self
    # @param manager ޥ͡㡦֥
    #
    # @else
    # @brief Constructor
    #
    # @endif
    def __init__(self, manager):
      self._manager = manager


    ##
    # @if jp
    # @brief λ
    #
    # ORBޥ͡㽪λ򳫻Ϥ롣
    #
    # @param self
    #
    # @else
    #
    # @endif
    def terminate(self):
      self._manager.shutdown()



  ##
  # @if jp
  # @class Term
  # @brief Term 饹
  #
  # λѥإѡ饹
  #
  # @since 0.4.0
  #
  # @else
  #
  # @endif
  class Term:



    def __init__(self):
      self.waiting = 0
      self.mutex   = threading.RLock()
